
/*
 * process.cc
 * Copyright (C) 1999 by John Heidemann
 * $Id: process_view.cc,v 1.23 1999/08/07 06:31:09 johnh Exp $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <stdio.h>  // snprintf

#include "main.hh"
#include "process_view.hh"
#include "process_model.hh"


bool
process_view_ordering::operator()(process_view *a, process_view *b) const
{
	// For some reason we get called with a null...
	// if we do reorders() in age().
	// Recover from that.
	if (!a || !b)
		return false;
	if (a->age_.last() != b->age_.last())
		return a->age_.last() < b->age_.last();
	if (a->size_.last() != b->size_.last())
		return a->size_.last() < b->size_.last();
	// force a total ordering
	return a->pm_->pid() < b->pm_->pid();
}


process_view::process_view_order_t process_view::orders_;


process_view::process_view(process_model *pm) :
	pm_(pm),
	order_(orders_.end())
{
	ENTRY_TRACE(__FILE__,__LINE__);
	size_.tick_set(scale_size(pm_->virtual_size_.last()));
	time_.tick_set(scale_time(pm_->utime_.last() + pm_->stime_.last()));
	b_ = blob::create_real_blob(this);
	reset_color();
	init_position();
	b_->set_size(size_.last());
}

process_view::~process_view()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	if (order_ != orders_.end())
		orders_.erase(order_);
	order_ = orders_.end();
	delete b_;
}

void
process_view::reorder()
{
	// optimize for not moving
	process_view_order_t::iterator prev_order_(order_);
	if (order_ != orders_.end()) {
		--prev_order_;
		orders_.erase(order_);
	};
	order_ = orders_.insert(prev_order_, this);
	// xxx: if the order changed, we now need
	// to signal the blob to redraw.
	// We currently lack that API.
	// Instead, count on it taking effect "next time".
}

void
process_view::reset_color()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	S_ = 0.6;
	B_ = 0.8 - (pm_->age() * 0.06);
	const double MIN_B = 0.15;
	if (B_ < MIN_B)
		B_ = MIN_B;

	H_ = (pm_->cmd_hash_.last() % 1024) / 1024.0;

	b_->set_hsb(H_, S_, B_);
}

void
process_view::init_position()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	int x = pm_->pid_ % (blob::X_MAX - blob::INITIAL_SLOP);
	int y = pm_->uid_ % (blob::Y_MAX - blob::INITIAL_SLOP);
	b_->set_position(x, y);
}

int
process_view::scale_size(int size)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	return size / 20;  // xxx: should by dynamic
}

int
process_view::scale_time(int time)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	// covert from usec to 100ths of a second
	return time / 10000;
}

void
process_view::update()
{
	ENTRY_TRACE(__FILE__,__LINE__);
	age_.tick_set(pm_->age());
	if (age_.changed() || pm_->cmd_hash_.changed())
		reset_color();

	if (report_vm.changed() ||
	    (report_vm.last() ? pm_->virtual_size_.changed() : pm_->resident_.changed())) {
		int size = scale_size(report_vm.last() ? 
					  pm_->virtual_size_.last() :
					  pm_->resident_.last());
		size = max(size, 3);   // work around not handling small sizes well.
		size_.tick_set(size);
		if (size_.changed())
			b_->grow(size_.change());
	};

	time_.tick_set(scale_time(pm_->utime_.last() + pm_->stime_.last()));
	if (time_.changed())
		b_->move(time_.change());

	if (age_.changed() || size_.changed())
		reorder();
}


void
process_view::info_to_buf(char *buf, int len)
{
	ENTRY_TRACE(__FILE__,__LINE__);
	// xxx: should be snprintf
	snprintf(buf, len, "user: %d\npid: %d\ncmd: %s\nnice: %d\nmem: %dKB\n",
		 pm_->uid_,
		 pm_->pid_, pm_->cmd_.c_str(), pm_->nice_, pm_->virtual_size_.last());
}

process_view *
process_view::superior()
{
	if (order_ == orders_.end())
	    return NULL;
	// I don't know how to check for moving our iterator
	// off the front.  I assumed it would return end(),
	// but maybe not.  The next check is paranoia.
	if (order_ == orders_.begin())
	    return NULL;
	process_view_order_t::iterator superior(order_);
	--superior;
	return superior == orders_.end() ? NULL : *superior;
}
